/* Copyright (C) |Meso|Star> 2015-2018 (contact@meso-star.com)
 *
 * This software is a program whose purpose is to test the spp library.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software. You can use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors have only limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and, more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms. */

#include "ssp.h"
#include "test_ssp_utils.h"

#include <rsys/clock_time.h>

#include <string.h>

#define NRAND 1024

static void /* Really basic test */
test_rng(const struct ssp_rng_type* type)
{
  FILE* stream;
  struct ssp_rng_type type2;
  struct ssp_rng* rng = NULL;
  struct ssp_rng* rng1 = NULL;
  struct ssp_rng* rng2 = NULL;
  struct mem_allocator allocator;
  struct time t0, t1;
  uint64_t datai0[NRAND];
  uint64_t datai1[NRAND];
  double datad[NRAND];
  float dataf[NRAND];
  char buf[512];
  int i, j;
  res_T r;
  const char can_set = (type != &ssp_rng_random_device);
  const char can_rw = (type != &ssp_rng_random_device);
  const char can_have_entropy = (type == &ssp_rng_random_device);
  const char can_discard = (type != &ssp_rng_random_device);

  mem_init_proxy_allocator(&allocator, &mem_default_allocator);

  CHK(type != NULL);
  CHK(ssp_rng_create(NULL, NULL, NULL) == RES_BAD_ARG);
  CHK(ssp_rng_create(&allocator, NULL, NULL) == RES_BAD_ARG);
  CHK(ssp_rng_create(NULL, type, NULL) == RES_BAD_ARG);
  CHK(ssp_rng_create(&allocator, type, NULL) == RES_BAD_ARG);
  CHK(ssp_rng_create(NULL, NULL, &rng) == RES_BAD_ARG);
  CHK(ssp_rng_create(&allocator, NULL, &rng) == RES_BAD_ARG);
  r = ssp_rng_create(NULL, type, &rng);
#ifdef WITH_R123_AES
  if (r == RES_BAD_OP && type == &ssp_rng_aes) {
    printf("AES-NI instructions not available on this CPU and system.\n");
    mem_shutdown_proxy_allocator(&allocator);
    check_memory_allocator(&allocator);
    return;
  }
#endif
  CHK(r == RES_OK);

  CHK(ssp_rng_get_type(NULL, NULL) == RES_BAD_ARG);
  CHK(ssp_rng_get_type(rng, NULL) == RES_BAD_ARG);
  CHK(ssp_rng_get_type(NULL, &type2) == RES_BAD_ARG);
  CHK(ssp_rng_get_type(rng, &type2) == RES_OK);
  CHK(ssp_rng_type_eq(type, &type2) == 1);

  CHK(ssp_rng_ref_get(NULL) == RES_BAD_ARG);
  CHK(ssp_rng_ref_get(rng) == RES_OK);
  CHK(ssp_rng_ref_put(NULL) == RES_BAD_ARG);
  CHK(ssp_rng_ref_put(rng) == RES_OK);
  CHK(ssp_rng_ref_put(rng) == RES_OK);

  CHK(ssp_rng_create(&allocator, type, &rng) == RES_OK);
  CHK(ssp_rng_set(NULL, 0) == RES_BAD_ARG);
  CHK(ssp_rng_set(rng, 0) == (can_set ? RES_OK : RES_BAD_OP));

  CHK(ssp_rng_create(NULL, type, &rng1) == RES_OK);
  CHK(ssp_rng_create(NULL, type, &rng2) == RES_OK);
  CHK(ssp_rng_discard(rng1, 10) == (can_discard ? RES_OK : RES_BAD_OP));
  if (type != &ssp_rng_random_device) {
    for (i = 0; i < 10; i++) ssp_rng_get(rng2);
    CHK(ssp_rng_get(rng1) == ssp_rng_get(rng2));
  }

  FOR_EACH(i, 0, NRAND) {
    datai0[i] = ssp_rng_get(rng);
    CHK(datai0[i] >= ssp_rng_min(rng));
    CHK(datai0[i] <= ssp_rng_max(rng));
    FOR_EACH(j, 0, i) {
      CHK(datai0[i] != datai0[j]);
    }
  }

  CHK(ssp_rng_set(rng, 0xDECAFBAD) == (can_set ? RES_OK : RES_BAD_OP));
  FOR_EACH(i, 0, NRAND) {
    datai1[i] = ssp_rng_get(rng);
    CHK(datai1[i] != datai0[i]);
    CHK(datai1[i] >= ssp_rng_min(rng));
    CHK(datai1[i] <= ssp_rng_max(rng));
    FOR_EACH(j, 0, i) {
      CHK(datai1[i] != datai1[j]);
    }
  }

  FOR_EACH(i, 0, NRAND) {
    datai0[i] = ssp_rng_uniform_uint64(rng, 1, 79);
    CHK(datai0[i] >= 1);
    CHK(datai0[i] <= 79);
  }
  FOR_EACH(i, 0, NRAND) {
    FOR_EACH(j, 0, NRAND) if(datai0[i] != datai0[j]) break;
    CHK(j != NRAND);
  }

  FOR_EACH(i, 0, NRAND) {
    datad[i] = ssp_rng_uniform_double(rng, -5.0, 11.3);
    CHK(datad[i] >= -5.0);
    CHK(datad[i] <= 11.3);
  }
  FOR_EACH(i, 0, NRAND) {
    FOR_EACH(j, 0, NRAND) if(datad[i] != datad[j]) break;
    CHK(j != NRAND);
  }

  FOR_EACH(i, 0, NRAND) {
    dataf[i] = ssp_rng_uniform_float(rng, -1.0, 12.5);
    CHK(dataf[i] >= -1.0);
    CHK(dataf[i] <= 12.5);
  }
  FOR_EACH(i, 0, NRAND) {
    FOR_EACH(j, 0, NRAND) if(dataf[i] != dataf[j]) break;
    CHK(j != NRAND);
  }

  FOR_EACH(i, 0, NRAND) {
    datad[i] = ssp_rng_canonical(rng);
    CHK(datad[i] >= 0.0);
    CHK(datad[i] < 1.0 );
    FOR_EACH(j, 0, i) {
      CHK(datad[i] != datad[j]);
    }
  }

  FOR_EACH(i, 0, NRAND) {
    dataf[i] = ssp_rng_canonical_float(rng);
    CHK(dataf[i] >= 0.0);
    CHK(dataf[i] < 1.0);
    FOR_EACH(j, 0, i) {
      CHK(dataf[i] != dataf[j]);
    }
  }

  time_current(&t0);
  FOR_EACH(i, 0, 1000000) {
    ssp_rng_get(rng);
  }
  time_current(&t1);
  time_sub(&t0, &t1, &t0);
  time_dump(&t0, TIME_SEC|TIME_MSEC|TIME_USEC, NULL, buf, sizeof(buf));
  printf("1,000,000 random numbers in %s\n", buf);

  if (can_have_entropy) {
    printf("Entropy for this implementation and system: %f\n",
      ssp_rng_entropy(rng));
  }

  stream = tmpfile();
  CHK(stream != NULL);
  CHK(ssp_rng_write(NULL, NULL) == RES_BAD_ARG);
  CHK(ssp_rng_write(rng, NULL) == RES_BAD_ARG);
  CHK(ssp_rng_write(NULL, stream) == RES_BAD_ARG);
  CHK(ssp_rng_write(rng, stream) == (can_rw ? RES_OK : RES_BAD_OP));

  FOR_EACH(i, 0, NRAND)
    datai0[i] = ssp_rng_get(rng);

  CHK(ssp_rng_read(NULL, NULL) == RES_BAD_ARG);
  CHK(ssp_rng_read(rng, NULL) == RES_BAD_ARG);
  CHK(ssp_rng_read(NULL, stream) == RES_BAD_ARG);
  CHK(ssp_rng_read(rng, stream) == (can_rw ? RES_IO_ERR : RES_BAD_OP));
  rewind(stream);
  CHK(ssp_rng_read(rng, stream) == (can_rw ? RES_OK : RES_BAD_OP));

  if (can_rw) {
    FOR_EACH(i, 0, NRAND) {
      uint64_t rn = ssp_rng_get(rng);
      CHK(rn == datai0[i]);
    }
  }
  fclose(stream);

  CHK(ssp_rng_ref_put(rng) == RES_OK);
  ssp_rng_ref_put(rng1);
  ssp_rng_ref_put(rng2);

  check_memory_allocator(&allocator);
  mem_shutdown_proxy_allocator(&allocator);
}

int
main(int argc, char** argv)
{
  if (argc <= 1) {
#ifdef WITH_R123_AES
    fprintf(stderr, "Usage: %s <kiss|mt19937_64|ranlux48|random_device|threefry|aes>\n", argv[0]);
#else
    fprintf(stderr, "Usage: %s <kiss|mt19937_64|ranlux48|random_device|threefry>\n", argv[0]);
#endif
    exit(0);
  }
  if(!strcmp(argv[1], "kiss")) {
    test_rng(&ssp_rng_kiss);
  } else if(!strcmp(argv[1], "mt19937_64")) {
    test_rng(&ssp_rng_mt19937_64);
  } else if(!strcmp(argv[1], "ranlux48")) {
    test_rng(&ssp_rng_ranlux48);
  } else if(!strcmp(argv[1], "random_device")) {
    test_rng(&ssp_rng_random_device);
  } else if(!strcmp(argv[1], "threefry")) {
    test_rng(&ssp_rng_threefry);
#ifdef WITH_R123_AES
  } else if(!strcmp(argv[1], "aes")) {
    test_rng(&ssp_rng_aes);
#endif
  } else {
    fprintf(stderr, "Unknown RNG `%s'\n", argv[1]);
    ASSERT(0);
  }

  CHK(mem_allocated_size() == 0);
  return 0;
}

